@@ -815,27 +815,4 @@ public class ZooKeeperTest extends ClientBase {
         assertThrows(IllegalArgumentException.class, () -> KeeperException.create(Code.get(Integer.MAX_VALUE)));
     }
 
-    @Test
-    public void testInvalidWatcherRegistration() throws Exception {
-        String nullWatcherMsg = "Invalid Watcher, shouldn't be null!";
-        final ZooKeeper zk = createClient();
-        try {
-            zk.register(null);
-            fail("Should validate null watcher!");
-        } catch (IllegalArgumentException iae) {
-            assertEquals(nullWatcherMsg, iae.getMessage());
-        }
-        try {
-            new ZooKeeper(hostPort, 10000, null, false);
-            fail("Should validate null watcher!");
-        } catch (IllegalArgumentException iae) {
-            assertEquals(nullWatcherMsg, iae.getMessage());
-        }
-        try {
-            new ZooKeeper(hostPort, 10000, null, 10L, "".getBytes(), false);
-            fail("Should validate null watcher!");
-        } catch (IllegalArgumentException iae) {
-            assertEquals(nullWatcherMsg, iae.getMessage());
-        }
-    }
 }
