@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -19,12 +19,14 @@
 package org.apache.zookeeper.test;
 
 import static org.apache.zookeeper.client.FourLetterWordMain.send4LetterWord;
-
-import java.io.BufferedInputStream;
-import java.io.ByteArrayOutputStream;
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertFalse;
+import static org.junit.jupiter.api.Assertions.assertTrue;
+import static org.junit.jupiter.api.Assertions.fail;
 import java.io.File;
-import java.io.FileInputStream;
 import java.io.IOException;
+import java.net.ConnectException;
+import java.net.ProtocolException;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Iterator;
@@ -36,13 +38,8 @@ import java.util.Set;
 import java.util.concurrent.CountDownLatch;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.TimeoutException;
-
 import javax.management.MBeanServerConnection;
 import javax.management.ObjectName;
-
-
-import org.apache.zookeeper.common.Time;
-import org.apache.zookeeper.common.X509Exception.SSLContextException;
 import org.apache.zookeeper.KeeperException;
 import org.apache.zookeeper.PortAssignment;
 import org.apache.zookeeper.TestableZooKeeper;
@@ -51,32 +48,31 @@ import org.apache.zookeeper.Watcher;
 import org.apache.zookeeper.Watcher.Event.KeeperState;
 import org.apache.zookeeper.ZKTestCase;
 import org.apache.zookeeper.ZooKeeper;
-import org.apache.zookeeper.common.IOUtils;
+import org.apache.zookeeper.client.ZKClientConfig;
+import org.apache.zookeeper.common.Time;
+import org.apache.zookeeper.common.X509Exception.SSLContextException;
 import org.apache.zookeeper.server.ServerCnxnFactory;
-import org.apache.zookeeper.server.ServerCnxnFactoryAccessor;
 import org.apache.zookeeper.server.ZKDatabase;
 import org.apache.zookeeper.server.ZooKeeperServer;
 import org.apache.zookeeper.server.persistence.FilePadding;
-import org.apache.zookeeper.server.persistence.FileTxnLog;
 import org.apache.zookeeper.server.quorum.QuorumPeer;
 import org.apache.zookeeper.server.util.OSMXBean;
-import org.junit.After;
-import org.junit.Assert;
-import org.junit.Before;
+import org.junit.jupiter.api.AfterEach;
+import org.junit.jupiter.api.BeforeEach;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 public abstract class ClientBase extends ZKTestCase {
+
     protected static final Logger LOG = LoggerFactory.getLogger(ClientBase.class);
 
     public static int CONNECTION_TIMEOUT = 30000;
-    static final File BASETEST =
-        new File(System.getProperty("build.test.dir", "build"));
 
     protected String hostPort = "127.0.0.1:" + PortAssignment.unique();
     protected int maxCnxns = 0;
     protected ServerCnxnFactory serverFactory = null;
     protected File tmpDir = null;
+    protected boolean exceptionOnFailedConnect = false;
 
     long initialFdCount;
 
@@ -84,18 +80,9 @@ public abstract class ClientBase extends ZKTestCase {
         super();
     }
 
-    /**
-     * In general don't use this. Only use in the special case that you
-     * want to ignore results (for whatever reason) in your test. Don't
-     * use empty watchers in real code!
-     *
-     */
-    protected static class NullWatcher implements Watcher {
-        public void process(WatchedEvent event) { /* nada */ }
-    }
-
     public static class CountdownWatcher implements Watcher {
-        // XXX this doesn't need to be volatile! (Should probably be final)
+
+        // TODO this doesn't need to be volatile! (Should probably be final)
         volatile CountDownLatch clientConnected;
         // Set to true when connected to a read-only server, or a read-write (quorum) server.
         volatile boolean connected;
@@ -107,13 +94,13 @@ public abstract class ClientBase extends ZKTestCase {
         public CountdownWatcher() {
             reset();
         }
-        synchronized public void reset() {
+        public synchronized void reset() {
             clientConnected = new CountDownLatch(1);
             connected = false;
             syncConnected = false;
             readOnlyConnected = false;
         }
-        synchronized public void process(WatchedEvent event) {
+        public synchronized void process(WatchedEvent event) {
             KeeperState state = event.getState();
             if (state == KeeperState.SyncConnected) {
                 connected = true;
@@ -134,15 +121,13 @@ public abstract class ClientBase extends ZKTestCase {
                 clientConnected.countDown();
             }
         }
-        synchronized public boolean isConnected() {
+        public synchronized boolean isConnected() {
             return connected;
         }
-        synchronized public void waitForConnected(long timeout)
-            throws InterruptedException, TimeoutException
-        {
+        public synchronized void waitForConnected(long timeout) throws InterruptedException, TimeoutException {
             long expire = Time.currentElapsedTime() + timeout;
             long left = timeout;
-            while(!connected && left > 0) {
+            while (!connected && left > 0) {
                 wait(left);
                 left = expire - Time.currentElapsedTime();
             }
@@ -151,12 +136,10 @@ public abstract class ClientBase extends ZKTestCase {
 
             }
         }
-        synchronized public void waitForSyncConnected(long timeout)
-                throws InterruptedException, TimeoutException
-        {
+        public synchronized void waitForSyncConnected(long timeout) throws InterruptedException, TimeoutException {
             long expire = Time.currentElapsedTime() + timeout;
             long left = timeout;
-            while(!syncConnected && left > 0) {
+            while (!syncConnected && left > 0) {
                 wait(left);
                 left = expire - Time.currentElapsedTime();
             }
@@ -164,12 +147,10 @@ public abstract class ClientBase extends ZKTestCase {
                 throw new TimeoutException("Failed to connect to read-write ZooKeeper server.");
             }
         }
-        synchronized public void waitForReadOnlyConnected(long timeout)
-                throws InterruptedException, TimeoutException
-        {
+        public synchronized void waitForReadOnlyConnected(long timeout) throws InterruptedException, TimeoutException {
             long expire = System.currentTimeMillis() + timeout;
             long left = timeout;
-            while(!readOnlyConnected && left > 0) {
+            while (!readOnlyConnected && left > 0) {
                 wait(left);
                 left = expire - System.currentTimeMillis();
             }
@@ -177,12 +158,10 @@ public abstract class ClientBase extends ZKTestCase {
                 throw new TimeoutException("Failed to connect in read-only mode to ZooKeeper server.");
             }
         }
-        synchronized public void waitForDisconnected(long timeout)
-            throws InterruptedException, TimeoutException
-        {
+        public synchronized void waitForDisconnected(long timeout) throws InterruptedException, TimeoutException {
             long expire = Time.currentElapsedTime() + timeout;
             long left = timeout;
-            while(connected && left > 0) {
+            while (connected && left > 0) {
                 wait(left);
                 left = expire - Time.currentElapsedTime();
             }
@@ -191,50 +170,42 @@ public abstract class ClientBase extends ZKTestCase {
 
             }
         }
+
     }
 
-    protected TestableZooKeeper createClient()
-        throws IOException, InterruptedException
-    {
+    protected TestableZooKeeper createClient() throws IOException, InterruptedException {
         return createClient(hostPort);
     }
 
-    protected TestableZooKeeper createClient(String hp)
-        throws IOException, InterruptedException
-    {
+    protected TestableZooKeeper createClient(String hp) throws IOException, InterruptedException {
         CountdownWatcher watcher = new CountdownWatcher();
         return createClient(watcher, hp);
     }
 
-    protected TestableZooKeeper createClient(CountdownWatcher watcher)
-        throws IOException, InterruptedException
-    {
+    protected TestableZooKeeper createClient(CountdownWatcher watcher) throws IOException, InterruptedException {
         return createClient(watcher, hostPort);
     }
 
-    private LinkedList<ZooKeeper> allClients;
+    private List<ZooKeeper> allClients;
     private boolean allClientsSetup = false;
 
-    protected TestableZooKeeper createClient(CountdownWatcher watcher, String hp)
-        throws IOException, InterruptedException
-    {
+    protected TestableZooKeeper createClient(CountdownWatcher watcher, String hp) throws IOException, InterruptedException {
         return createClient(watcher, hp, CONNECTION_TIMEOUT);
     }
 
-    protected TestableZooKeeper createClient(CountdownWatcher watcher,
-            String hp, int timeout)
-        throws IOException, InterruptedException
-    {
+    protected TestableZooKeeper createClient(CountdownWatcher watcher, String hp, int timeout) throws IOException, InterruptedException {
         watcher.reset();
         TestableZooKeeper zk = new TestableZooKeeper(hp, timeout, watcher);
-        if (!watcher.clientConnected.await(timeout, TimeUnit.MILLISECONDS))
-        {
-            Assert.fail("Unable to connect to server");
+        if (!watcher.clientConnected.await(timeout, TimeUnit.MILLISECONDS)) {
+            if (exceptionOnFailedConnect) {
+                throw new ProtocolException("Unable to connect to server");
+            }
+            fail("Unable to connect to server");
         }
-        synchronized(this) {
+        synchronized (this) {
             if (!allClientsSetup) {
                 LOG.error("allClients never setup");
-                Assert.fail("allClients never setup");
+                fail("allClients never setup");
             }
             if (allClients != null) {
                 allClients.add(zk);
@@ -249,25 +220,27 @@ public abstract class ClientBase extends ZKTestCase {
     }
 
     public static class HostPort {
+
         String host;
         int port;
         public HostPort(String host, int port) {
             this.host = host;
             this.port = port;
         }
+
     }
     public static List<HostPort> parseHostPortList(String hplist) {
         ArrayList<HostPort> alist = new ArrayList<HostPort>();
-        for (String hp: hplist.split(",")) {
+        for (String hp : hplist.split(",")) {
             int idx = hp.lastIndexOf(':');
             String host = hp.substring(0, idx);
             int port;
             try {
                 port = Integer.parseInt(hp.substring(idx + 1));
-            } catch(RuntimeException e) {
+            } catch (RuntimeException e) {
                 throw new RuntimeException("Problem parsing " + hp + e.toString());
             }
-            alist.add(new HostPort(host,port));
+            alist.add(new HostPort(host, port));
         }
         return alist;
     }
@@ -283,10 +256,12 @@ public abstract class ClientBase extends ZKTestCase {
                 // if there are multiple hostports, just take the first one
                 HostPort hpobj = parseHostPortList(hp).get(0);
                 String result = send4LetterWord(hpobj.host, hpobj.port, "stat", secure);
-                if (result.startsWith("Zookeeper version:") &&
-                        !result.contains("READ-ONLY")) {
+                if (result.startsWith("Zookeeper version:") && !result.contains("READ-ONLY")) {
                     return true;
                 }
+            } catch (ConnectException e) {
+                // ignore as this is expected, do not log stacktrace
+                LOG.info("server {} not up: {}", hp, e.toString());
             } catch (IOException e) {
                 // ignore as this is expected
                 LOG.info("server {} not up", hp, e);
@@ -337,8 +312,7 @@ public abstract class ClientBase extends ZKTestCase {
     /**
      * Return true if any of the states is achieved
      */
-    public static boolean waitForServerState(QuorumPeer qp, int timeout,
-            String... serverStates) {
+    public static boolean waitForServerState(QuorumPeer qp, int timeout, String... serverStates) {
         long start = Time.currentElapsedTime();
         while (true) {
             try {
@@ -357,33 +331,32 @@ public abstract class ClientBase extends ZKTestCase {
         }
     }
 
-    static void verifyThreadTerminated(Thread thread, long millis)
-        throws InterruptedException
-    {
+    static void verifyThreadTerminated(Thread thread, long millis) throws InterruptedException {
         thread.join(millis);
         if (thread.isAlive()) {
-            LOG.error("Thread " + thread.getName() + " : "
-                    + Arrays.toString(thread.getStackTrace()));
-            Assert.assertFalse("thread " + thread.getName()
-                    + " still alive after join", true);
+            LOG.error("Thread {} : {}", thread.getName(), Arrays.toString(thread.getStackTrace()));
+            assertFalse(true, "thread " + thread.getName() + " still alive after join");
         }
     }
 
     public static File createEmptyTestDir() throws IOException {
-        return createTmpDir(BASETEST, false);
+        return createTmpDir(testBaseDir, false);
     }
 
     public static File createTmpDir() throws IOException {
-        return createTmpDir(BASETEST, true);
+        return createTmpDir(testBaseDir, true);
     }
 
     static File createTmpDir(File parentDir, boolean createInitFile) throws IOException {
+        if (!parentDir.exists()) {
+            parentDir.mkdir();
+        }
         File tmpFile = File.createTempFile("test", ".junit", parentDir);
         // don't delete tmpFile - this ensures we don't attempt to create
         // a tmpDir with a duplicate name
         File tmpDir = new File(tmpFile + ".dir");
-        Assert.assertFalse(tmpDir.exists()); // never true if tmpfile does it's job
-        Assert.assertTrue(tmpDir.mkdirs());
+        assertFalse(tmpDir.exists()); // never true if tmpfile does it's job
+        assertTrue(tmpDir.mkdirs());
 
         // todo not every tmp directory needs this file
         if (createInitFile) {
@@ -396,13 +369,13 @@ public abstract class ClientBase extends ZKTestCase {
     public static void createInitializeFile(File dir) throws IOException {
         File initFile = new File(dir, "initialize");
         if (!initFile.exists()) {
-            Assert.assertTrue(initFile.createNewFile());
+            assertTrue(initFile.createNewFile());
         }
     }
 
     private static int getPort(String hostPort) {
         String[] split = hostPort.split(":");
-        String portstr = split[split.length-1];
+        String portstr = split[split.length - 1];
         String[] pc = portstr.split("/");
         if (pc.length > 1) {
             portstr = pc[0];
@@ -413,16 +386,19 @@ public abstract class ClientBase extends ZKTestCase {
     /**
      * Starting the given server instance
      */
-    public static void startServerInstance(File dataDir,
-            ServerCnxnFactory factory, String hostPort, int serverId) throws IOException,
-            InterruptedException {
+    public static void startServerInstance(
+            File dataDir,
+            ServerCnxnFactory factory,
+            String hostPort,
+            int serverId) throws IOException, InterruptedException {
         final int port = getPort(hostPort);
         LOG.info("STARTING server instance 127.0.0.1:{}", port);
         ZooKeeperServer zks = new ZooKeeperServer(dataDir, dataDir, 3000);
         zks.setCreateSessionTrackerServerId(serverId);
         factory.startup(zks);
-        Assert.assertTrue("waiting for server up", ClientBase.waitForServerUp(
-                "127.0.0.1:" + port, CONNECTION_TIMEOUT, factory.isSecure()));
+        assertTrue(
+                ClientBase.waitForServerUp("127.0.0.1:" + port, CONNECTION_TIMEOUT, factory.isSecure()),
+                "waiting for server up");
     }
 
     /**
@@ -432,16 +408,17 @@ public abstract class ClientBase extends ZKTestCase {
      * Because any exception on starting the server would leave the server
      * running and the caller would not be able to shutdown the instance. This
      * may affect other test cases.
-     * 
+     *
      * @return newly created server instance
-     * 
+     *
      * @see <a
      *      href="https://issues.apache.org/jira/browse/ZOOKEEPER-1852">ZOOKEEPER-1852</a>
      *      for more information.
      */
     public static ServerCnxnFactory createNewServerInstance(
-            ServerCnxnFactory factory, String hostPort, int maxCnxns)
-            throws IOException, InterruptedException {
+            ServerCnxnFactory factory,
+            String hostPort,
+            int maxCnxns) throws IOException, InterruptedException {
         final int port = getPort(hostPort);
         LOG.info("CREATING server instance 127.0.0.1:{}", port);
         if (factory == null) {
@@ -450,13 +427,11 @@ public abstract class ClientBase extends ZKTestCase {
         return factory;
     }
 
-    static void shutdownServerInstance(ServerCnxnFactory factory,
-            String hostPort)
-    {
+    static void shutdownServerInstance(ServerCnxnFactory factory, String hostPort) {
         if (factory != null) {
             ZKDatabase zkDb = null;
             {
-                ZooKeeperServer zs = getServer(factory);
+                ZooKeeperServer zs = factory.getZooKeeperServer();
                 if (zs != null) {
                     zkDb = zs.getZKDatabase();
                 }
@@ -471,10 +446,9 @@ public abstract class ClientBase extends ZKTestCase {
             }
             final int PORT = getPort(hostPort);
 
-            Assert.assertTrue("waiting for server down",
-                       ClientBase.waitForServerDown("127.0.0.1:" + PORT,
-                                                    CONNECTION_TIMEOUT,
-                                                    factory.isSecure()));
+            assertTrue(
+                    ClientBase.waitForServerDown("127.0.0.1:" + PORT, CONNECTION_TIMEOUT, factory.isSecure()),
+                    "waiting for server down");
         }
     }
 
@@ -484,7 +458,7 @@ public abstract class ClientBase extends ZKTestCase {
     public static void setupTestEnv() {
         // during the tests we run with 100K prealloc in the logs.
         // on windows systems prealloc of 64M was seen to take ~15seconds
-        // resulting in test Assert.failure (client timeout on first session).
+        // resulting in test failure (client timeout on first session).
         // set env and directly in order to handle static init/gc issues
         System.setProperty("zookeeper.preAllocSize", "100");
         FilePadding.setPreallocSize(100 * 1024);
@@ -495,7 +469,7 @@ public abstract class ClientBase extends ZKTestCase {
         allClientsSetup = true;
     }
 
-    @Before
+    @BeforeEach
     public void setUp() throws Exception {
         setUpWithServerId(1);
     }
@@ -507,19 +481,20 @@ public abstract class ClientBase extends ZKTestCase {
          * only place sun has implemented as part of the mgmt bean api.
          */
         OSMXBean osMbean = new OSMXBean();
-        if (osMbean.getUnix() == true) {
-            initialFdCount = osMbean.getOpenFileDescriptorCount();  	
-            LOG.info("Initial fdcount is: "
-                    + initialFdCount);
+        if (osMbean.getUnix()) {
+            initialFdCount = osMbean.getOpenFileDescriptorCount();
+            LOG.info("Initial fdcount is: {}", initialFdCount);
         }
 
         setupTestEnv();
 
+        setupCustomizedEnv();
+
         JMXEnv.setUp();
 
         setUpAll();
 
-        tmpDir = createTmpDir(BASETEST, true);
+        tmpDir = createTmpDir(testBaseDir, true);
 
         startServer(serverId);
 
@@ -530,14 +505,17 @@ public abstract class ClientBase extends ZKTestCase {
         startServer(1);
     }
 
+    /**
+     * Give it a chance to set up customized env before starting the server.
+     */
+    public void setupCustomizedEnv() { /* do nothing by default */ }
+
     private void startServer(int serverId) throws Exception {
         LOG.info("STARTING server");
-        serverFactory = createNewServerInstance(serverFactory, hostPort,
-                maxCnxns);
+        serverFactory = createNewServerInstance(serverFactory, hostPort, maxCnxns);
         startServerInstance(tmpDir, serverFactory, hostPort, serverId);
         // ensure that server and data bean are registered
-        Set<ObjectName> children = JMXEnv.ensureParent("InMemoryDataTree",
-                "StandaloneServer_port");
+        Set<ObjectName> children = JMXEnv.ensureParent("InMemoryDataTree", "StandaloneServer_port");
         // Remove beans which are related to zk client sessions. Strong
         // assertions cannot be done for these client sessions because
         // registeration of these beans with server will happen only on their
@@ -551,24 +529,22 @@ public abstract class ClientBase extends ZKTestCase {
                 Iterator<ObjectName> childItr = children.iterator();
                 while (childItr.hasNext()) {
                     ObjectName clientBean = childItr.next();
-                    if (clientBean.toString().contains(
-                            getHexSessionId(zkc.getSessionId()))) {
-                        LOG.info("found name:" + zkc.getSessionId()
-                                + " client bean:" + clientBean.toString());
+                    if (clientBean.toString().contains(getHexSessionId(zkc.getSessionId()))) {
+                        LOG.info("found name:{} client bean:{}", zkc.getSessionId(), clientBean.toString());
                         childItr.remove();
                     }
                 }
             }
         }
         for (ObjectName bean : children) {
-            LOG.info("unexpected:" + bean.toString());
+            LOG.info("unexpected:{}", bean.toString());
         }
-        Assert.assertEquals("Unexpected bean exists!", 0, children.size());
+        assertEquals(0, children.size(), "Unexpected bean exists!");
     }
 
     /**
      * Returns a string representation of the given long value session id
-     * 
+     *
      * @param sessionId
      *            long value of session id
      * @return string representation of session id
@@ -585,28 +561,24 @@ public abstract class ClientBase extends ZKTestCase {
         JMXEnv.ensureOnly();
     }
 
-
-    protected static ZooKeeperServer getServer(ServerCnxnFactory fac) {
-        ZooKeeperServer zs = ServerCnxnFactoryAccessor.getZkServer(fac);
-
-        return zs;
-    }
-
     protected void tearDownAll() throws Exception {
         synchronized (this) {
-            if (allClients != null) for (ZooKeeper zk : allClients) {
-                try {
-                    if (zk != null)
-                        zk.close();
-                } catch (InterruptedException e) {
-                    LOG.warn("ignoring interrupt", e);
+            if (allClients != null) {
+                for (ZooKeeper zk : allClients) {
+                    try {
+                        if (zk != null) {
+                            zk.close();
+                        }
+                    } catch (InterruptedException e) {
+                        LOG.warn("ignoring interrupt", e);
+                    }
                 }
             }
             allClients = null;
         }
     }
 
-    @After
+    @AfterEach
     public void tearDown() throws Exception {
         LOG.info("tearDown starting");
 
@@ -615,7 +587,7 @@ public abstract class ClientBase extends ZKTestCase {
         stopServer();
 
         if (tmpDir != null) {
-            Assert.assertTrue("delete " + tmpDir.toString(), recursiveDelete(tmpDir));
+            assertTrue(recursiveDelete(tmpDir), "delete " + tmpDir.toString());
         }
 
         // This has to be set to null when the same instance of this class is reused between test cases
@@ -629,10 +601,9 @@ public abstract class ClientBase extends ZKTestCase {
          * only place sun has implemented as part of the mgmt bean api.
          */
         OSMXBean osMbean = new OSMXBean();
-        if (osMbean.getUnix() == true) {
-            long fdCount = osMbean.getOpenFileDescriptorCount();     
-            String message = "fdcount after test is: "
-                    + fdCount + " at start it was " + initialFdCount;
+        if (osMbean.getUnix()) {
+            long fdCount = osMbean.getOpenFileDescriptorCount();
+            String message = "fdcount after test is: " + fdCount + " at start it was " + initialFdCount;
             LOG.info(message);
             if (fdCount > initialFdCount) {
                 LOG.info("sleeping for 20 secs");
@@ -640,23 +611,27 @@ public abstract class ClientBase extends ZKTestCase {
                 //assertTrue(message, fdCount <= initialFdCount);
             }
         }
+
+        cleanUpCustomizedEnv();
     }
 
+    public void cleanUpCustomizedEnv() { /* do nothing by default */ }
+
     public static MBeanServerConnection jmxConn() throws IOException {
         return JMXEnv.conn();
     }
 
     public static boolean recursiveDelete(File d) {
-       return TestUtils.deleteFileRecursively(d, true);
+        return TestUtils.deleteFileRecursively(d, true);
     }
 
     public static void logAllStackTraces() {
         StringBuilder sb = new StringBuilder();
         sb.append("Starting logAllStackTraces()\n");
         Map<Thread, StackTraceElement[]> threads = Thread.getAllStackTraces();
-        for (Entry<Thread, StackTraceElement[]> e: threads.entrySet()) {
+        for (Entry<Thread, StackTraceElement[]> e : threads.entrySet()) {
             sb.append("Thread " + e.getKey().getName() + "\n");
-            for (StackTraceElement elem: e.getValue()) {
+            for (StackTraceElement elem : e.getValue()) {
                 sb.append("\tat " + elem + "\n");
             }
         }
@@ -668,17 +643,15 @@ public abstract class ClientBase extends ZKTestCase {
      * Verify that all of the servers see the same number of nodes
      * at the root
      */
-    void verifyRootOfAllServersMatch(String hostPort)
-        throws InterruptedException, KeeperException, IOException
-    {
-        String parts[] = hostPort.split(",");
+    void verifyRootOfAllServersMatch(String hostPort) throws InterruptedException, KeeperException, IOException {
+        String[] parts = hostPort.split(",");
 
         // run through till the counts no longer change on each server
         // max 15 tries, with 2 second sleeps, so approx 30 seconds
         int[] counts = new int[parts.length];
         int failed = 0;
         for (int j = 0; j < 100; j++) {
-            int newcounts[] = new int[parts.length];
+            int[] newcounts = new int[parts.length];
             int i = 0;
             for (String hp : parts) {
                 try {
@@ -691,15 +664,14 @@ public abstract class ClientBase extends ZKTestCase {
                     }
                 } catch (Throwable t) {
                     failed++;
-                    // if session creation Assert.fails dump the thread stack
+                    // if session creation fails dump the thread stack
                     // and try the next server
                     logAllStackTraces();
                 }
             }
 
             if (Arrays.equals(newcounts, counts)) {
-                LOG.info("Found match with array:"
-                        + Arrays.toString(newcounts));
+                LOG.info("Found match with array:{}", Arrays.toString(newcounts));
                 counts = newcounts;
                 break;
             } else {
@@ -707,7 +679,7 @@ public abstract class ClientBase extends ZKTestCase {
                 Thread.sleep(10000);
             }
 
-            // don't keep this up too long, will Assert.assert false below
+            // don't keep this up too long, will assert false below
             if (failed > 10) {
                 break;
             }
@@ -716,38 +688,14 @@ public abstract class ClientBase extends ZKTestCase {
         // verify all the servers reporting same number of nodes
         String logmsg = "node count not consistent{} {}";
         for (int i = 1; i < parts.length; i++) {
-            if (counts[i-1] != counts[i]) {
-                LOG.error(logmsg, Integer.valueOf(counts[i-1]), Integer.valueOf(counts[i]));
+            if (counts[i - 1] != counts[i]) {
+                LOG.error(logmsg, counts[i - 1], counts[i]);
             } else {
-                LOG.info(logmsg, Integer.valueOf(counts[i-1]), Integer.valueOf(counts[i]));
+                LOG.info(logmsg, counts[i - 1], counts[i]);
             }
         }
     }
 
-    public static String readFile(File file) throws IOException {
-        ByteArrayOutputStream os = new ByteArrayOutputStream();
-        BufferedInputStream is = new BufferedInputStream(new FileInputStream(file));
-        try {
-            IOUtils.copyBytes(is, os, 1024, true);
-        } finally {
-            is.close();
-        }
-        return os.toString();
-    }
-
-    public static String join(String separator, Object[] parts) {
-        StringBuilder sb = new StringBuilder();
-        boolean first = true;
-        for (Object part : parts) {
-            if (!first) {
-                sb.append(separator);
-                first = false;
-            }
-            sb.append(part);
-        }
-        return sb.toString();
-    }
-
     public static ZooKeeper createZKClient(String cxnString) throws Exception {
         return createZKClient(cxnString, CONNECTION_TIMEOUT);
     }
@@ -766,14 +714,21 @@ public abstract class ClientBase extends ZKTestCase {
         return createZKClient(cxnString, sessionTimeout, CONNECTION_TIMEOUT);
     }
 
-    public static ZooKeeper createZKClient(String cxnString, int sessionTimeout, long connectionTimeout) throws IOException {
+    public static ZooKeeper createZKClient(String cxnString, int sessionTimeout,
+        long connectionTimeout) throws IOException {
+        return createZKClient(cxnString, sessionTimeout, connectionTimeout, new ZKClientConfig());
+    }
+
+    public static ZooKeeper createZKClient(String cxnString, int sessionTimeout,
+        long connectionTimeout, ZKClientConfig config) throws IOException {
         CountdownWatcher watcher = new CountdownWatcher();
-        ZooKeeper zk = new ZooKeeper(cxnString, sessionTimeout, watcher);
+        ZooKeeper zk = new ZooKeeper(cxnString, sessionTimeout, watcher, config);
         try {
             watcher.waitForConnected(connectionTimeout);
         } catch (InterruptedException | TimeoutException e) {
-            Assert.fail("ZooKeeper client can not connect to " + cxnString);
+            fail("ZooKeeper client can not connect to " + cxnString);
         }
         return zk;
     }
+
 }
